;********************************************************************
;          implementace 1-wire slave do mikroprocesoru AVR
;    (c) 2005 Ales Povalac, alpov@inMail.cz, http://www.alpov.net/
;********************************************************************


;********************************************************************
; hlavni program
;********************************************************************
; kalibrace oscilatoru, watchdog, doplnujici fce atd.
; tento kod pravdepodobne budete menit k dosazeni pozadovane funkce
;********************************************************************


.include "tn26def.inc"		; definice ATtiny26

.eseg	; pamet EEPROM

ROM_ADDR:
.db	254,1,0,0,0,0,0,12	; ROM adresa zarizeni (posledni bajt DOW-CRC!)

OUTPUTS:
.db	0b00111111		; vystupy na PORTA

TESTOUT:
.db	0b00000011		; testovaci vystup aktivovany tlacitkem

.cseg	; pamet FLASH

.org	0x0000
	rjmp	MCURESET	; obsluha resetu

.org	INT0addr
	rjmp	INT_EXT0	; obsluha testovaciho tlacitka

.org	OVF0addr
	rjmp	OW_INT_OVF0	; obsluha preteceni casovace


.equ	VERSION = 100		; verze programu

.equ	OWDATA = PORTB5		; 1-wire pin

.def	itemp = r25		; registry
.def	icount = r24

.def	A = r16
.def	rwtemp = r17
.def	data = r18
.def	count = r19
.def	addr = r20
.def	temp = r21
.def	crc = r22


.org	FLASHEND - 1	; kalibrace, prepisovane pri programovani (avrsp -c)

CAL_TABLE:
.db	0x7F,0x7F	; kalibracni bajty oscilatoru pro 8 MHz a 4 MHz
.db	0x7F,0x7F	; kalibracni bajty oscilatoru pro 2 MHz a 1 MHz

.org	0x0010

;--------------------------------------------------------------------
; INT_EXT0 - obsluha testovaciho tlacitka
;--------------------------------------------------------------------
INT_EXT0:
	ldi	addr,TESTOUT
	out	EEAR,addr
	ldi	A,(1<<EERE)
	out	EECR,A
	in	A,EEDR		; precist testovaci vystup z EEPROM
	out	PORTA,A		; odeslat na PORTA
INT_EXT0_1:
	wdr
	rjmp	INT_EXT0_1	; nekonecna smycka, ceka reset


;--------------------------------------------------------------------
; MCURESET - volane po resetu mikroprocesoru
;--------------------------------------------------------------------
MCURESET:
	ldi	ZH,HIGH(CAL_TABLE<<1)
	ldi	ZL,LOW(CAL_TABLE<<1)+1	; pro 4 MHz druhy od konce
	lpm	A,Z			; kalibracni konstanta z tabulky
	out	OSCCAL,A		; kalibruj oscilator

	wdr
	ldi	A,(1<<WDE)|(1<<WDP1)|(1<<WDP0)
	out	WDTCR,A			; povolit watchdog na cca 0.1sec

	ldi	A,(1<<ISC01)
	out	MCUCR,A			; interrupt na sestupnou hranu (detekce tlacitka)
	ldi	A,(1<<INT0)
	out	GIMSK,A			; povolit externi preruseni

	rjmp	OW_INIT			; inicializace a spusteni owslave obsluhy


.include "owslave.inc"		; implementace nizkourovnovych funkci 1-wire sbernice
.include "rom_cmd.inc"		; obsluha ROM prikazu
.include "func_cmd.inc"		; obsluha funkcniho prikazu
.include "dowcrc.inc"		; vypocet DOW-CRC
