;********************************************************************
;          implementace 1-wire slave do mikroprocesoru AVR
;    (c) 2005 Ales Povalac, alpov@inMail.cz, http://www.alpov.net/
;********************************************************************


;********************************************************************
; implementace nizkourovnovych funkci 1-wire sbernice
;********************************************************************
; vyuziva casovac:	casovac 0, preteceni na OW_INT_OVF0
; vyuziva registry:	A, icount, itemp, rwtemp, data, count 
; inicializace:		OW_INIT
; spusteni:		OW_WAIT4RESET
;********************************************************************


.equ	T_RST_PDH = 30		; casovani komunikace
.equ	T_RST_PDL = 120
.equ	T_SAMPLE = 30
.equ	XTAL = 4		; krystal

;--------------------------------------------------------------------
; DELAY - makro cekani
;--------------------------------------------------------------------
.macro	DELAY
	ldi	A,(@0*XTAL)/3 - @1/3
DELAY_1:
	dec	A		; 1
	brne	DELAY_1		; 2
.endmacro


;--------------------------------------------------------------------
; WAIT4LOW - makro cekani na 0 na sbernici
;--------------------------------------------------------------------
.macro	WAIT4LOW
WAIT4LOW_1:
	wdr			; resetovat watchdog
	sbic	PINB,OWDATA
	rjmp	WAIT4LOW_1	; dokud 1 tak znovu
.endmacro


;--------------------------------------------------------------------
; WAIT4HIGH - makro cekani na 1 na sbernici
;--------------------------------------------------------------------
.macro	WAIT4HIGH
WAIT4HIGH_1:
	wdr			; resetovat watchdog
	sbis	PINB,OWDATA
	rjmp	WAIT4HIGH_1	; dokud 0 tak znovu
.endmacro


;--------------------------------------------------------------------
; OW_INT_OVF0 - obsluha preteceni casovace
;--------------------------------------------------------------------
OW_INT_OVF0:
	dec	icount		; pocitadlo
	brne	OW_INT_OVF0_1	; konec pokud neni 0
	ldi	itemp,0
	out	TCCR0,itemp	; zastavit citac
	sbic	PINB,OWDATA	; pokud je linka na 1 tak navrat
OW_INT_OVF0_1:
	reti
	rjmp	OW_PRESENCE	; jinak reset


;--------------------------------------------------------------------
; OW_READBIT - cteni bitu ze sbernice
;--------------------------------------------------------------------
OW_READBIT:
	WAIT4HIGH
	WAIT4LOW		; cekej na sestupnou hranu
	ldi	A,0
	out	TCNT0,A		; vynulovat casovac
	ldi	icount,XTAL
	ldi	A,(1<<CS00)
	out	TCCR0,A		; spustit citac
	DELAY	T_SAMPLE,9	; cekej T_SAMPLE
	clc
	sbic	PINB,OWDATA	; navrat = sbernice
	sec
	ret


;--------------------------------------------------------------------
; OW_WRITEBIT - zapis bitu C na sbernici
;--------------------------------------------------------------------
OW_WRITEBIT:
	ldi	A,0		; ne clr! smaze carry!
	ldi	rwtemp,(1<<CS00)
	WAIT4HIGH
	WAIT4LOW		; cekej na sestupnou hranu
	out	TCNT0,A		; vynulovat casovac
	ldi	icount,XTAL
	out	TCCR0,rwtemp	; spustit citac
	brcs	OW_WRITEBIT_1
	sbi	DDRB,OWDATA	; pokud neni carry, tak OWDATA na 0
OW_WRITEBIT_1:
	DELAY	T_SAMPLE,7	; cekej T_SAMPLE
	cbi	DDRB,OWDATA	; OWDATA zpatky
	ret


;--------------------------------------------------------------------
; OW_READBYTE - cteni bajtu ze sbernice
;--------------------------------------------------------------------
OW_READBYTE:
	clr	data
	ldi	count,8
OW_READBYTE_1:
	rcall	OW_READBIT
	ror	data
	dec	count
	brne	OW_READBYTE_1	; cteni osmi bitu
	ret


;--------------------------------------------------------------------
; OW_WRITEBYTE - zapis bajtu data na sbernici
;--------------------------------------------------------------------
OW_WRITEBYTE:
	ldi	count,8
OW_WRITEBYTE_1:
	ror	data
	rcall	OW_WRITEBIT
	dec	count
	brne	OW_WRITEBYTE_1	; zapis osmi bitu
	ret


;--------------------------------------------------------------------
; OW_WAIT4RESET - cekani na reset
;--------------------------------------------------------------------
OW_WAIT4RESET:	
	clr	A
	out	TCCR0,A		; zastavit citac
	out	TCNT0,A		; vynulovat citac
	ldi	icount,XTAL
	WAIT4LOW
	ldi	A,(1<<CS00)	; spustit citac
	out	TCCR0,A
	WAIT4HIGH
	rjmp	OW_WAIT4RESET	; nekonecna smycka (ceka na interrupt)
	

;--------------------------------------------------------------------
; OW_PRESENCE - reakce na reset odeslanim presence pulsu
;--------------------------------------------------------------------
OW_PRESENCE:	
	ldi	A,RAMEND	; vycistit zasobnik
	out	SP,A
	sei			; povolit preruseni
	clr	A
	out	TCCR0,A		; zastavit citac
	out	TCNT0,A		; vynulovat citac
	ldi	icount,XTAL
	WAIT4HIGH		; cekej na 1
	DELAY	T_RST_PDH,0	; cekej T_RST_PDH
	sbi	DDRB,OWDATA	; OWDATA = 0
	DELAY	T_RST_PDL,0	; cekej T_RST_PDL
	cbi	DDRB,OWDATA	; OWDATA jako vstup
	rjmp	ROM_CMD		; skok na zpracovani ROM prikazu


;--------------------------------------------------------------------
; OW_INIT - inicializace
;--------------------------------------------------------------------
OW_INIT:
	ldi	A,(1<<TOIE0)	; povolit preruseni od casovace 0
	out	TIMSK,A
	ldi	A,RAMEND	; vycistit zasobnik
	out	SP,A
	rcall	FUNC_CMD_INIT	; inicializace
	cbi	DDRB,OWDATA	; OWDATA tri-state
	cbi	PORTB,OWDATA
	sei			; povolit globalni preruseni
	rjmp	OW_WAIT4RESET
