;********************************************************************
;          implementace 1-wire slave do mikroprocesoru AVR
;    (c) 2005 Ales Povalac, alpov@inMail.cz, http://www.alpov.net/
;********************************************************************


;********************************************************************
; obsluha ROM prikazu
;********************************************************************
; vyuziva registry:	A, temp, addr, data, count 
; inicializace:		neni
;********************************************************************


.equ	CODE_ROM_READ = 0x33	; 1-wire prikazy
.equ	CODE_ROM_MATCH = 0x55
.equ	CODE_ROM_SKIP = 0xCC
.equ	CODE_ROM_SEARCH = 0xF0


;--------------------------------------------------------------------
; ROM_CMD - zpracovani ROM prikazu
;--------------------------------------------------------------------
ROM_CMD:
	rcall	OW_READBYTE		; cteni ROM prikazu
	cpi	data,CODE_ROM_READ
	breq	ROM_CMD_READ		; READ
	cpi	data,CODE_ROM_MATCH
	breq	ROM_CMD_MATCH		; MATCH
	cpi	data,CODE_ROM_SKIP
	breq	FUNC_CMD		; SKIP
	cpi	data,CODE_ROM_SEARCH
	breq	ROM_CMD_SEARCH		; SEARCH
	rjmp	OW_WAIT4RESET		; jinak cekej na reset


;--------------------------------------------------------------------
; ROM_CMD_READ - cteni ROM adresy
;--------------------------------------------------------------------
ROM_CMD_READ:
	ldi	addr,ROM_ADDR
ROM_CMD_READ_1:
	out	EEAR,addr
	ldi	A,(1<<EERE)
	out	EECR,A
	in	data,EEDR	; precist bajt adresy z EEPROM
	rcall	OW_WRITEBYTE	; odeslat
	inc	addr
	cpi	addr,8
	brne	ROM_CMD_READ_1	; vsech 8 bajtu
	rjmp	FUNC_CMD	; cekej na prikaz


;--------------------------------------------------------------------
; ROM_CMD_MATCH - kontrola ROM adresy
;--------------------------------------------------------------------
ROM_CMD_MATCH:
	ldi	addr,ROM_ADDR
ROM_CMD_MATCH_1:
	out	EEAR,addr
	rcall	OW_READBYTE	; precist bajt ze sbernice
	ldi	A,(1<<EERE)
	out	EECR,A
	in	A,EEDR		; precist bajt adresy z EEPROM
	cp	A,data		; porovnat
	breq	ROM_CMD_MATCH_2
	rjmp	OW_WAIT4RESET	; kdyz se lisi tak bez cekat na reset
ROM_CMD_MATCH_2:
	inc	addr
	cpi	addr,8
	brne	ROM_CMD_MATCH_1
	rjmp	FUNC_CMD	; cekej na prikaz


;--------------------------------------------------------------------
; ROM_CMD_SEARCH - vyhledavani zarizeni na sbernici
;--------------------------------------------------------------------
ROM_CMD_SEARCH:
	ldi	addr,ROM_ADDR
ROM_CMD_SEARCH_1:
	out	EEAR,addr
	ldi	A,(1<<EERE)
	out	EECR,A
	in	data,EEDR	; precist bajt adresy z EEPROM

	ldi	count,8		; 8 bitu
ROM_CMD_SEARCH_2:
	ror	data		; postupne bereme vsech 64 bitu
	in	temp,SREG
	rcall	OW_WRITEBIT	; zapsat bit
	ldi	A,(1<<SREG_C)
	eor	temp,A
	out	SREG,temp	; negovat carry
	rcall	OW_WRITEBIT	; zapsat negovany bit
	
	rcall	OW_READBIT	; precist bit
	in	A,SREG
	eor	A,temp
	sbrs	A,SREG_C	; porovnani precteneho a zapsaneho bitu
	rjmp	OW_WAIT4RESET	; pokud jsou ruzne tak cekej na reset

	dec	count
	brne	ROM_CMD_SEARCH_2	; vsech 8 bitu

	inc	addr
	cpi	addr,8
	brne	ROM_CMD_SEARCH_1	; ze vsech 8 bajtu
	rjmp	FUNC_CMD		; cekej na prikaz
